
; Expected contents of input_info:
;   input_info.l_fid          L-channel input file ID
;   input_info.temp.min       Lower temperature for spectral library
;   input_info.temp.max       Upper temperature for spectral library
;   input_info.emiss.min      Lower emissivity for spectral library
;   input_info.emiss.max      Upper emissivity for spectral library
;   input_info.sl_outfile     Name of file for output of the spectral library

function l_data_sim, input_info

aux_location = oat_addon_location(subdir='readomega_aux')
e = ENVI(/CURRENT)

status = {status: 0, msg: 'OK'}     ; default status structure

; The raster, which we only need for its header's dmars value
l_raster = ENVIFIDToRaster(input_info.l_fid)

if ~l_raster.metadata.hastag ('dmars') then begin
  ; We needed that
  message, 'L-data raster does not contain Mars distance info (dmars).'
endif

dmars = l_raster.metadata['dmars']

; Prepare to sample Planck function
waves=fltarr(352)
openr,lun, aux_location+'omega_waves_all_mod.txt',/get_lun
readf,lun,waves
free_lun, lun

wavelengths_l = waves(224:351) ; waves is ordered V, C, L
num_bands = 128

T_1 = input_info.temp.min
T_2 = input_info.temp.max
epsilon_1 = input_info.emiss.min
epsilon_2 = input_info.emiss.max

; output spectra named for which T (1 vs 2) and which epsilon (1 vs 2), respectively
result_sp_1_1 = fltarr(num_bands)
result_sp_1_2 = fltarr(num_bands)
result_sp_2_1 = fltarr(num_bands)
result_sp_2_2 = fltarr(num_bands)

; Sample Planck function to build radiance spectra
for b = 0L, num_bands-1 do begin
  wv = wavelengths_l[b]
  result_sp_1_1[b] = planck_function_radiance(wv,T_1,epsilon_1)
  result_sp_1_2[b] = planck_function_radiance(wv,T_1,epsilon_2)
  result_sp_2_1[b] = planck_function_radiance(wv,T_2,epsilon_1)
  result_sp_2_2[b] = planck_function_radiance(wv,T_2,epsilon_2)
endfor

specsol=fltarr(352)
openr, lun, aux_location+'specsol_0403.dat', /get_lun
readf, lun, specsol
free_lun, lun

specsol_l = specsol(128:255) ; specsol is ordered C, L, V

; Correct with heliocentric distance in AU
specsol_l = specsol_l/dmars/dmars

; Change results from radiance into I/F
result_sp_1_1 /= specsol_l
result_sp_1_2 /= specsol_l
result_sp_2_1 /= specsol_l
result_sp_2_2 /= specsol_l

; Save spectral library to disk and display

sli_data = fltarr(num_bands,4)
sli_data[*,0] = result_sp_1_1
sli_data[*,1] = result_sp_1_2
sli_data[*,2] = result_sp_2_1
sli_data[*,3] = result_sp_2_2

; names
format_string = 'iof_T_%f_emiss_%f'
sli_names = [ string(format=format_string,T_1,epsilon_1), $
              string(format=format_string,T_1,epsilon_2), $
              string(format=format_string,T_2,epsilon_1), $
              string(format=format_string,T_2,epsilon_2) ]

save_spectral_library, sli_data, wavelengths_l, sli_names, input_info.sl_outfile

; Bring the spectral library viewer into focus with the new library open in it
; TODO: this is really bad form. ENVI doesn't have anything CLOSE to as good as the raster
; tools for spectral libraries, so here we are, running an invalid command because it brings
; about a useful side effect.
foo = e.OpenRaster(input_info.sl_outfile, error=err)

return, status

end
