pro oat_atmo_corr
  COMPILE_OPT IDL2

  catch, err
  if err ne 0 then begin
    catch, /cancel
    void = dialog_message('Could not perform operation. An error occurred:' + $
      !error_state.msg, /error)
    message, /reset
    return
  endif

  e = envi(/current)
  UI = e.UI
  
  ; This operation requires the C & L channel I/F for an OMEGA scene
  
  ; Get C & L cubes using ENVI standard dialog with no subsetting allowed
  
  c_raster = UI.SelectInputData(/RASTER, /DISABLE_BANDS, /DISABLE_SUB_RECT, /DISABLE_MASK, /DISABLE_NO_DATA, SUB_RECT=subRect, title='Select OMEGA C-channel data:')
  if (c_raster eq !null) then return ; I guess we're not doing it, then
  fid_c = ENVIRasterToFID(c_raster)
  c_filename = c_raster.uri
  
  l_raster = UI.SelectInputData(/RASTER, /DISABLE_BANDS, /DISABLE_SUB_RECT, /DISABLE_MASK, /DISABLE_NO_DATA, SUB_RECT=subRect, title='Select OMEGA L-channel data:')
  if (l_raster eq !null) then return ; I guess we're not doing it, then
  fid_l = ENVIRasterToFID(l_raster)
  l_filename = l_raster.uri
  
  ; TODO: size-related consistency checks, and also checking if the orbit # is the same
  
  ; Get locations for C & L outputs via standard dialogs
  
  default_c_output = insert_file_suffix(c_filename, '_corr', /last_ext)
  default_l_output = insert_file_suffix(l_filename, '_corr', /last_ext)
  
  base = widget_auto_base(title='Corrected C-channel data')
  wo = widget_outf(base, uvalue='outf',default=default_c_output, /auto)
  output_prefs_c = auto_wid_mng(base)
  
  base = widget_auto_base(title='Corrected L-channel data')
  wo = widget_outf(base, uvalue='outf',default=default_l_output, /auto)
  output_prefs_l = auto_wid_mng(base)
  
  ; make the call
  
  correction_info = { c_in : { fid: fid_c, $
                               filename: c_filename }, $
                      l_in : { fid: fid_l, $
                               filename: l_filename }, $
                      c_out : { outfile: output_prefs_c.outf }, $
                      l_out : { outfile: output_prefs_l.outf } $
                    } 
  
  stat = atmospheric_correction(correction_info,/progress_bar)
  
  if (stat.status lt 0) then begin
    ; Fatal error problem to report:
    ok = dialog_message(stat.msg, title='Fatal Error', /error)
  endif else if (stat.status gt 0) then begin
    ; Non-fatal problem to report:
    ok = dialog_message(stat.msg, title='Warning')
  endif

end
