
pro oat_iof_map_project
  COMPILE_OPT IDL2

  catch, err
  if err ne 0 then begin
    catch, /cancel
    void = dialog_message('Could not perform operation. An error occurred:' + $
      !error_state.msg, /error)
    message, /reset
    return
  endif

  e = envi(/current)
  UI = e.UI

  ; The basic version: separately get a NAV and a qube and throw 'em in there
  
  ; Get NAV and a qube using ENVI standard dialog with no subsetting allowed

  data_raster = UI.SelectInputData(/RASTER, /DISABLE_BANDS, /DISABLE_SUB_RECT, /DISABLE_MASK, /DISABLE_NO_DATA, SUB_RECT=subRectData, title='Select an OMEGA data cube:')
  if (data_raster eq !null) then return ; I guess we're not doing it, then
  fid_data = ENVIRasterToFID(data_raster)
  data_filename = data_raster.uri

  nav_raster = UI.SelectInputData(/RASTER, /DISABLE_BANDS, /DISABLE_SUB_RECT, /DISABLE_MASK, /DISABLE_NO_DATA, SUB_RECT=subRectNav, title='Select the corresponding NAV cube:')
  if (nav_raster eq !null) then return ; I guess we're not doing it, then
  fid_nav = ENVIRasterToFID(nav_raster)
  nav_filename = nav_raster.uri

  ; TODO: size-related consistency checks, and also checking if the orbit # is the same
  ; TODO: accept all stuff at once (C,V,L (or any subset), plus NAV), and then sort it out and do them all?

  ; Get location for projected cube via standard dialog
  default_output_filename = insert_file_suffix(data_filename, '_proj', /last_ext)
  base = widget_auto_base(title='Projected data cube')
  wo = widget_outf(base, uvalue='outf',default=default_output_filename, /auto)
  pix_size_widget = widget_param(base, prompt='Output pixel size (m)', default = '600.00', dt=4, uvalue='outsz', /auto)
  output_prefs = auto_wid_mng(base)
  
  if (output_prefs.accept eq 0) then return ; I guess we're not doing it, then
  
  pix_size = output_prefs.outsz
  
  dims = [-1, subRectData[0], subRectData[2], subRectData[1], subRectData[3]]

  ; What channel is this data? Find out and choose geo bands
  wvs = data_raster.metadata['wavelength']
  min_wv = min(wvs)
  max_wv = max(wvs)
  
  ; NOTE: if data file is a weird subset that fits in the overlap, the wrong channel
  ; might be chosen (but that is a far-fetched thing to do)
  
  ; These values from omega_waves_all_mod.txt are rounded outward to nearest 0.001 um
  if min_wv ge 0.926 and max_wv le 2.695 then begin
    channel = 'C'
  endif else if min_wv ge 2.527 and max_wv le 5.089 then begin
    channel = 'L'
  endif else if min_wv ge 0.350 and max_wv le 1.071 then begin
    channel = 'V'
  endif else begin
    ; TODO: we really oughta die with error msg here
  endelse
  
  ema_band = 5 ; The ema rel. to gravity field, to imitate how CAT uses the areoid-relative ema for this
  if channel eq 'C' then begin
     lat_band = 7
     lon_band = 6
  endif else if channel eq 'L' then begin
    lat_band = 22
    lon_band = 21
  endif else if channel eq 'V' then begin
    lat_band = 37
    lon_band = 36
  endif

  proj_info = { image: { $
                  dims: dims,$
                  fid: fid_data,$
                  bands: indgen(data_raster.NBANDS) },$
                output: { $
                  outfile: output_prefs.outf,$
                  pixsize: pix_size},$
                nav: { $
                  fid: fid_nav,$
                  lat_band: lat_band,$
                  lon_band: lon_band,$
                  ema_band: ema_band     }}

  status = omega_map_projection(proj_info)
  
end
