;******************************************************************************
;
; function crism_reference_coordinates, lat, lon, ema, reflat, reflon, $
;           min_ema_row
;
; Find the reference latitude & longitude for projection in an OMEGA Observation.
; Based on crism_reference_coordinates.pro in the CRISM Analysis Tool.
;
; INPUT PARAMETERS:
;	lat       fltarr[ns,nl]   Latitudes from NAV Cube
;	lon       fltarr[ns,nl]   Longitudes from NAV Cube
;	ema       fltarr[ns,nl]   Emission angles from NAV Cube
;
; OUTPUT PARAMETERS:
;	reflat         float   Reference latitude
;	reflon         float   Reference longitude
;	min_ema_row    int     Index of the row containing the minimum emission angle
;
; RETURN
;	Status: 0, success; -1, no valid data
;
; KEYWORDS: none
;
; NOTES:
;	Reference lat & lon from center column of row with minimum emission angle
;******************************************************************************

function omega_reference_coordinates, lat, lon, ema, reflat, reflon, $
           min_ema_row

errflag = 0

sz = size(ema)
ns = sz[1]
nl = sz[2]

;; Min EMA:
;k = where((ema lt 65000.0), nk)    ; avoid CRISM NaN
;if (nk eq 0) then begin
;	errflag = -1
;	return, errflag
;endif
;minema = min(ema[k], minidx)
minema = min(ema, minidx)
; Min EMA row:
min_ema_row = minidx[0] / ns

; Reference latitude:
reflat = lat[ns/2, min_ema_row]
reflon = lon[ns/2, min_ema_row]

return, errflag
end

