function planck_function, wavelength_um, temp
  ; Expects wavelength_um in, well, micrometers
  ; Expects temp in Kelvins
  ; Returns: planck function value in J * m^-2 * s^-1 * sr^-1 * um^-1
  
  ; Wavelength conversion to m
  wavelength = wavelength_um / 1.0e6
  
  h = 6.626e-34 ; Planck constant, J*s
  k_b = 1.3806e-23 ; Boltzmann constant, J/K
  c = 3.0e8 ; speed of light in vacuum, m/s
  
  planck_numerator = 2*h*c^2
  planck_denominator = wavelength^5 * ( exp(h*c/(k_b*wavelength*temp)) - 1 )
  
  planck_function_value = planck_numerator / planck_denominator
  
  ; That value is in J * m^-3 * s^-1 * sr^-1, so a bit of conversion to get um in there
  planck_function_value = planck_function_value / 1.0e6
  
  return, planck_function_value

end

; Function to get radiance related to the Planck function value at a given temperature and wavelength
function planck_function_radiance, wavelength, temp, emissivity
  ; Expects wavelength in um
  ; Expects temp in Kelvins
 
  radiance = emissivity * planck_function(wavelength, temp)
  return, radiance
 
end
