
; Readomega aux functions

; A function that takes 1-indexed default bands and a boolean array of
; bad-bands-list values (1 for good, 0 for bad) and returns a 1-indexed default
; bands set that only has good spectels in it.
function adjustDefaultBands, defBandsIn, goodSpectelList
  defBandsOut = defBandsIn
  
  numBands = n_elements(goodSpectelList)

  for i=0,2 do begin ; for each default band
    
    ; If this default band is marked bad, bump to the next one until one works
    while (defBandsOut[i] le numBands) && (~goodSpectelList[defBandsOut[i]-1]) do begin
      defBandsOut[i] += 1
    endwhile
    
    ; Did the loop end b/c of overflow? Well, that sucks
    if defBandsOut[i] gt numBands then begin
      lastGoodBandZeroIndexed = (where(goodSpectelList))[-1]
      if lastGoodBand eq -1 then begin
        ; Wow, there are 0 good spectels. That's a problem
        defBandsOut[i] = 1 ; TODO: Will this crash everything?
      endif else begin
        defBandsOut[i] = lastGoodBandZeroIndexed + 1
      endelse
    endif
    
  endfor
  
  return, defBandsOut
end



; Procedure to compute I/F for OMEGA scene
;
; Parameters:
; - data_qube_path: the full path to the raw .qub file (a different extension could cause a crash at the moment)
; - nav_file_path: the full path to the raw .nav file that corresponds to the given data qube
; - output_path: the folder to write the processed files to. Must include trailing path separator
;
pro readomega, data_qube_path, nav_file_path, output_path, PROGRESS_BAR=progress_bar

FORWARD_FUNCTION ENVIFIDTORASTER, ENVIRASTER, ENVIAbortable

aux_location = oat_addon_location(subdir='readomega_aux')

; progress bar setup
if keyword_set(progress_bar) then begin
  e = ENVI(/CURRENT)
  Channel = e.GetBroadcastChannel()
  Abort = ENVIAbortable()
  Start = ENVIStartMessage('Raw OMEGA Data to I/F', Abort)
  Channel.Broadcast, Start
  tstart = systime(1)
endif

; Note: other than the raster saving at the end, this file is based on OMEGA SOFT10

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Initializing', 0, Abort)
  Channel.Broadcast, Progress
endif

; Continuing the old naming conventions, at least for now
nomfic = data_qube_path ; C:\bar\orb9876_5.qub
nomgeo = nav_file_path ; C:\quux\orb9876_5.nav
nomfic0 = file_basename(data_qube_path) ; orb9876_5.qub
nomgeo0 = file_basename(nav_file_path) ; orb9876_5.nav
orbnum_str = file_basename(nomfic0,'.qub') ; orb9876_5

orbnum=omega_orbit(orbnum_str)

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Reading data', 1, Abort)
  Channel.Broadcast, Progress
endif

openr, lun_qube, nomfic, ERROR=errflag, /get_lun
free_lun, lun_qube
if (errflag ne 0) then begin
  print,'file ',nomfic,' not found'
  return
endif
readcube, nomfic, idat, sdat0, sdat1,info
if(size(idat))(0) ne 3 then begin
  print,'***** only one line in cube ',nomfic0,' ******'
  return
endif
exposure=0
exposure=info(0:2)
summation=info(3)
bits_per_data=info(4)
data_quality=fix(info(5))

openr, lun_geo, nomgeo, ERROR=errflag, /get_lun
if(errflag ne 0) then begin
  print,' no corresponding NAV cube'
  free_lun, lun_geo
  dmars=1.52
  specmars=fltarr(352)
  openr, lun, aux_location+'specsol_0403.dat', /get_lun
  readf, lun, specmars
  free_lun, lun
  specmars=specmars/dmars/dmars
  goto, nogeom
endif
trans= !pi/180.*1.e-4

geocube=0B
ilec=0B

read_geolbl,nomgeo, '', values, data

lrec   = data[0]
nrec   = data[1]
npixel = data[2]
npara  = data[3]
nscan  = data[4]
nau    = data[5]
dmars=nau*1.e-4
openr, lun, aux_location+'specsol_0403.dat', /get_lun
specmars=0B
specmars=fltarr(352)
readf, lun, specmars
free_lun, lun

specmars=specmars/dmars/dmars

data = 0B
geocube=0B

geocube = lonarr(npixel, npara, nscan)
ilec = lonarr(npixel)

point_lun, lun_geo, lrec*nrec

for  k=0,nscan-1 do begin
   for j=0,npara-1 do begin
      readu, lun_geo, ilec
      geocube(*,j,k) = ilec
   endfor
endfor
free_lun, lun_geo
if(geocube(1,1,0) gt 13) then geocube=swap_endian(temporary(geocube))
longi=reform(geocube(*,6,*)*1.e-4)
lati=reform(geocube(*,7,*)*1.e-4)
alt=reform(geocube(*,12,*)*1.e-3)
nogeom:
close,/all ; TODO: _is_ there anything left to close?
; preliminary pipeline tool

a=size(idat)
nbal=a(3)
npix=a(1)
jdat=0
jdat=float(idat)

fond2=intarr(256)
openr, lun, aux_location+'fond2.dat', /get_lun
readf, lun, fond2
free_lun, lun
if(exposure(1) gt 4.) then fond2=2*fond2
fondcur=fond2(128:255)#(0.*indgen(nbal)+1.)

jdat=0
jdat=float(idat)
i=where(idat le 1)
if(i(0) ne -1) then jdat(i)=1.e-5

pix0IR=0
i=where(idat(*,0:255,*) le 0)
if(i(0) ne -1) then pix0IR=(size(i))(3)
print,'       0 or less  IR: ',pix0IR

hkmin=min(sdat1(14,1,*))
if(hkmin lt 6) then hkmin=6
indj=where(sdat1(14,1,*) ge hkmin)
i6=indj(0)
indj=where(sdat1(14,1,*) ge hkmin+1)
balHK=indj(0)-i6
indi=where(sdat1(14,1,*) ge 6 and sdat0(10,*) gt 100)
balsm=balHK*8

if(balHK gt 0 and nbal gt indi(0)+balsm) then begin
  a=0.
  b=0.
  c=0.
  d=0.
  ndeb=indi(0)
  nf=(size(indi))(1)-1
  nfin=indi(nf)
  if(sdat0(26,nbal-1) lt 20) then sdat0(*,nbal-1)=sdat0(*,nbal-2)
  for k=0,255 do begin
    b=reform(float(sdat0(k,indi)))
    a=[2*b(0)-rotate(b(0:balsm-1),2),b,2*b(nf)-rotate(b(nf-balsm+1:nf),2)]
    c=(smooth(a,balsm))(balsm:balsm+nf)
    d= -sdat0(k,ndeb:nbal-1)+spline(indi,c,ndeb+indgen(nbal-ndeb))
    for i=0,npix-1 do jdat(i,k,ndeb:nbal-1)= $
         jdat(i,k,ndeb:nbal-1)+d
  endfor
endif

for i=0,npix-1 do jdat(i,128:255,*)=jdat(i,128:255,*)-fondcur

i=where(jdat lt 1.e-5)
if(i(0) ne -1) then jdat(i)=1.e-5

if(summation ne 1) then jdat=jdat/summation
linearC=0
linearC=fltarr(4096)
openr, lun, aux_location+'linearC.dat', /get_lun
readf, lun, linearC
free_lun, lun
jdat(*,0:127,*)=linearC(fix(jdat(*,0:127,*)+0.5))
wvl=0
wvl=fltarr(352)
openr, lun, aux_location+'lambda_0403.dat', /get_lun
readf, lun, wvl
free_lun, lun
mtf=fltarr(352)
rap=fltarr(3,256)
bound=intarr(3,256)
openr, lun, aux_location+'boundcur.dat', /get_lun
readf, lun, bound
free_lun, lun
if(exposure(1) lt 4.) then begin
  openr, lun, aux_location+'mtf120315_25.dat', /get_lun
  readf, lun, mtf
  free_lun, lun
  openr, lun, aux_location+'rapcur_25.dat', /get_lun
  readf, lun, rap
  free_lun, lun
endif else begin
  openr, lun, aux_location+'mtf120315_50.dat', /get_lun
  readf, lun, mtf
  free_lun, lun
  openr, lun, aux_location+'rapcur_50.dat', /get_lun
  readf, lun, rap
  free_lun, lun
endelse

ib=where(orbnum gt bound(0,*))
mtf(ib)=mtf(ib)*rap(0,ib)
ib=where(orbnum gt bound(1,*))
if(ib(0) ge 0) then mtf(ib)=mtf(ib)*rap(1,ib)
ib=where(orbnum gt bound(2,*))
if(ib(0) ge 0) then mtf(ib)=mtf(ib)*rap(2,ib)

; correction voie L
rapl=fltarr(128)
openr, lun, aux_location+'rapmtflcur.bin', /get_lun
tmp=FSTAT(lun)
orbmax=tmp.size/512-1
orbcur=orbnum
if(orbcur gt orbmax) then begin
  orbcur=orbmax
  print,orbcur,format='(" warning: L channel corrected as for orbit",I5)' 
endif
offs=long(512)*orbcur
point_LUN,lun,offs
readu,lun,rapl
free_lun, lun
mtf(128:255)=mtf(128:255)*rapl
if(exposure(1) gt 5.) then mtf(0:255)=mtf(0:255)/5.*exposure(1)

for n=0,255 do jdat(*,n,*)=jdat(*,n,*)/mtf(n)
ic=where(mtf lt 10000.)

;**************************************************************************************************************************
;bit error correction
;**************************************************************************************************************************

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Bit error correction', 15, Abort)
  Channel.Broadcast, Progress
endif

vis=float(idat(*,256:351,*))
siz = size(idat)
lines = siz(3)
pixels = siz(1)
exptime = info(2)/1000.
image_ratio = fltarr(pixels, lines)

level = 4095.*info(3)
if pixels EQ 128 then level = info(3)*4095*2.

i=where(vis le 0)
counter_neg=0
if(i(0) ne -1) then begin
  counter_neg=(size(i))(3)
  vis(i)=0.001
endif

i=where(vis gt level)
counter_pos=0
if(i(0) ne -1) then begin
  counter_pos=(size(i))(3)
  vis(i)=level
endif
i=where(vis le level and vis gt 0.8*level)
counter_sat=0
if(i(0) ne -1) then begin
  counter_sat=(size(i))(3)
  vis(i)=level*0.8
endif

plan3=reform(vis(*,3,*))
counter_spike3=0
i=where(plan3 gt 0.5*(vis(*,2,*)+vis(*,3,*))+50)
if(i(0) ne -1) then begin
  counter_spike3=(size(i))(3)
  plan3(i)=0.5*(reform(vis(*,2,*)+vis(*,3,*)))(i)
  vis(*,3,*)=plan3
endif

median_line = fltarr(lines)
column = fltarr(lines)

counter_spikes=long(0)

if lines ge 10 then begin
  ; despiking
  for k = 0, 95 do begin
      for m = 0, pixels - 1 do begin
         column(*) = vis(m, k, *)
         median_line = median(column, 7)
           for i = 0, lines-1 do begin
            if (abs(median_line(i) - vis(m, k, i)) GT 200) then begin
            vis(m, k, i) = median_line(i)
            counter_spikes = counter_spikes + 1
            endif
           endfor
      endfor
  endfor
endif

print, ' negative pixels VIS:', counter_neg
print, 'anomalous pixels VIS:', counter_pos
print, 'saturated pixels VIS:', counter_sat
print, '          spikes VIS:', counter_spikes

vis = float(fix(vis))

;******************************************************************************************************************************
; bias correction
;******************************************************************************************************************************

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Bias correction', 25, Abort)
  Channel.Broadcast, Progress
endif

percen = 3.4e-6
fper = 0.
pend = 5.
if npixel eq 128 then fper = 1.
if npixel eq 64 then fper = 2.
if npixel eq 32 then fper = 4.
if npixel eq 16 then fper = 8.

strL1 =0.
strL2 = fltarr(96)
Strl_nc = 0.

lv = wvl(256:*)
lft1 = linfit([lv(0), lv(95)], [percen*fper, (percen*fper) +(percen*fper)/100.*pend ])
Strl_nc = (lv*lft1(1)+lft1(0))

for j = 0, lines-1 do begin 
      for k = 0, 96-1 do strL2(k) = total(vis(*, k, j))
      strL1 = total(vis(*, *, j))
      for i = 0, pixels -1 do begin
          vis(i, *, j) = (vis(i, *, j) - 50.*strL2*percen*fper)- 0.75*strL1*strl_nc;       
      endfor
endfor

;******************************************************************************************************************************
; smear correction
;******************************************************************************************************************************

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Smear correction', 30, Abort)
  Channel.Broadcast, Progress
endif

for i = 0, pixels-1 do begin
    for n = 0, lines-1 do begin
       vis(i, *, n) = smear_corr_050701( vis(i, *, n), info(2))

    endfor
endfor


;******************************************************************************************************************************
;flat cube
;******************************************************************************************************************************

slice = ulonarr(128, 96)

openr, lun, aux_location+'flatVIS050701.bin', /get_lun
readu, lun, slice
free_lun, lun
if(slice(48,0) gt 40000) then slice=swap_endian(slice)


istart= (128 - pixels)/2.
iend  = istart + pixels - 1

for i = 0, lines-1 do begin

    vis(*, *, i) = vis(*, *, i) * 2.^15 / slice(istart:iend, *)

endfor

;***************************************************************************************************************************
;radiometric calibration
;***************************************************************************************************************************

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Radiometric calibration', 50, Abort)
  Channel.Broadcast, Progress
endif

f = 1.
if pixels eq 128 then  f = 2; internal summation
f2 = 1.
if exptime eq 0.1 then f2 = 2.
if exptime eq 0.2 then f2 = 4.

spectrum = fltarr(96)

for i = 0, lines - 1 do begin
    for m = 0, pixels - 1 do begin
       spectrum = vis(m, *, i)
       jdat(m, 256:351, i) = ordcorr(wvl(256:351), spectrum)/ ( f2 * mtf(256:351)) / f / summation
    endfor
    ;print, 'done %', 100*float(i)/(lines - 1)
endfor


;**************************************
;Convert to I/F by dividing by Specmars
;
;	Added by K.Lichtenberg 5-9-2006
;**************************************

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('I/F conversion', 65, Abort)
  Channel.Broadcast, Progress
endif

for j=0,lines-1 do begin
  for i=0,pixels-1 do begin
    jdat(i,*,j)=jdat(i,*,j)/specmars
  endfor
endfor

;*******************************************
;Reorder so VNIR on left and transpose array
;*******************************************

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Prepping for output', 70, Abort)
  Channel.Broadcast, Progress
endif

temp96=fltarr(96)
temp256=fltarr(256)

;now re-order the bands so VNIR is on the left
temp96 = jdat[*,256:351,*]
temp256 = jdat[*,0:255,*]
jdat[*,0:95,*]=temp96
jdat[*,96:351,*]=temp256

;transpose jdat and geocube
transqub = transpose(jdat,[0,2,1])
transgeo = transpose(geocube,[0,2,1])

transgeo = float(transgeo)
transgeo[*,*,2:5]   /= 10000.0 ; angles not associated with a channel
transgeo[*,*,6:10]  /= 10000.0 ; angles associated with C-channel, first part
transgeo[*,*,13:20] /= 10000.0 ; angles associated with C-channel, second part
transgeo[*,*,21:25] /= 10000.0 ; angles associated with L-channel, first part
transgeo[*,*,28:35] /= 10000.0 ; angles associated with L-channel, second part
transgeo[*,*,36:40] /= 10000.0 ; angles associated with V-channel, first part
transgeo[*,*,43:50] /= 10000.0 ; angles associated with V-channel, second part

transqub_c=transqub(*,*,96:223)
transqub_l=transqub(*,*,224:351)
transqub_v=transqub(*,*,0:95)

waves=fltarr(352)
openr,lun, aux_location+'omega_waves_all_mod.txt',/get_lun
readf,lun,waves
free_lun, lun

nav_waves=strarr(51)
openr,lun, aux_location+'nav_cube_waves.txt',/get_lun
readf,lun,nav_waves
free_lun, lun

; Where are the good spectels? The ones in the list ic above.
good_spectels = mtf lt 10000.

size_cube=size(transqub)
name_cube_c=output_path + nomfic0 + '_C.IMG'
name_cube_l=output_path + nomfic0 + '_L.IMG'
name_cube_v=output_path + nomfic0 + '_V.IMG'
size_geo=size(transgeo)
name_geo=output_path + nomfic0 + '_NAV.IMG'

if keyword_set(progress_bar) then begin
  Progress = ENVIProgressMessage('Writing outputs to disk', 75, Abort)
  Channel.Broadcast, Progress
endif

; TODO: really ought to address root cause of mysterious error msgs caught with emsg below

; Write C cube
c_raster = ENVIRaster(transqub_c,URI=name_cube_c)
c_raster.metadata.AddItem, 'wavelength', waves(96:223), error=emsg
c_raster.metadata.AddItem, 'wavelength units', 'Micrometers'
c_raster.metadata.AddItem, 'bbl', good_spectels[0:127]
c_raster.metadata.AddItem, 'data ignore value', 65535.0 ; not useful yet, but helps the right stuff show up immediately after map projection
c_raster.metadata.AddItem, 'default bands', adjustDefaultBands([115,41,10], good_spectels[0:127])
c_raster.metadata.AddItem, 'dmars', dmars ; Distance to the Sun at observation time, AU
c_raster.save

; Write L cube
l_raster = ENVIRaster(transqub_l,URI=name_cube_l)
l_raster.metadata.AddItem, 'wavelength', waves(224:351), error=emsg
l_raster.metadata.AddItem, 'wavelength units', 'Micrometers'
l_raster.metadata.AddItem, 'bbl', good_spectels[128:255]
l_raster.metadata.AddItem, 'data ignore value', 65535.0 ; not useful yet, but helps the right stuff show up immediately after map projection
l_raster.metadata.AddItem, 'default bands', adjustDefaultBands([58,72,24], good_spectels[128:255])
l_raster.metadata.AddItem, 'dmars', dmars ; Distance to the Sun at observation time, AU
l_raster.save

; Write V cube
v_raster = ENVIRaster(transqub_v,URI=name_cube_v)
v_raster.metadata.AddItem, 'wavelength', waves(0:95), error=emsg
v_raster.metadata.AddItem, 'wavelength units', 'Micrometers'
v_raster.metadata.AddItem, 'bbl', good_spectels[256:351]
v_raster.metadata.AddItem, 'data ignore value', 65535.0 ; not useful yet, but helps the right stuff show up immediately after map projection
v_raster.metadata.AddItem, 'default bands', adjustDefaultBands([48,34,25], good_spectels[256:351])
v_raster.metadata.AddItem, 'dmars', dmars ; Distance to the Sun at observation time, AU
v_raster.save

; Write geocube
geo_raster = ENVIRaster(transgeo,URI=name_geo)
geo_raster.metadata.AddItem, 'band names', nav_waves, error=emsg
geo_raster.save

; Display the data cubes
display_raster_current_view, v_raster
display_raster_current_view, c_raster
display_raster_current_view, l_raster

if keyword_set(progress_bar) then begin
  Finish = ENVIFinishMessage(Abort)
  Channel.Broadcast, Finish
  tnow = systime(1)
  print, 'Readomega time (s): ',tnow-tstart
endif

;these next two lines reverse the pixels and bands of the cube if they are backward.
;this should be taken out if the lat and lon are correct originally!!
;transqub = reverse(transqub,1,/OVERWRITE)	;for pixels (left-right)
;transqub = reverse(transqub,2,/OVERWRITE)	;for lines (north-south)

end
