
pro save_spectral_library, spectra, wvs, spNames, sl_filename
; Expects spectra to be a float array with dimensions: 
;   # wavelengths by # spectra.
; Wvs is expected to be in micrometers.
; SpNames is expected to be an array of # spectra strings.
; sl_filename is the file to which we'll save the library

spectraSz = size(spectra)
num_wvs = spectraSz[1]
num_sp = spectraSz[2]

openw, ofl, sl_filename, /get_lun
writeu, ofl, spectra
close, ofl
free_lun, ofl

; If the data file has no extension, the header adds ".hdr", and otherwise
; the extension (only the last if there are multiple) if replaced by ".hdr"
pos=strpos(sl_filename,'.',/reverse_search)
if (pos ne -1) then begin ; an extension exists
  fname_base = strmid(sl_filename,0,pos)
endif else begin ; no extension
  fname_base=sl_filename
endelse
spheader=string(format='%s.hdr',fname_base) ; TODO: this is gonna run into cold reality as soon as we try to run this on a system with case-sensitive filenames

openw, hdrf, spheader, /get_lun
printf, hdrf, 'ENVI'
printf, hdrf, 'description = {}'
printf, hdrf, 'samples = ', num_wvs
printf, hdrf, 'lines = ', num_sp
printf, hdrf, 'bands = 1'
printf, hdrf, 'header_offset = 0'
printf, hdrf, 'file type = ENVI Spectral Library'
printf, hdrf, 'data type = 4'
printf, hdrf, 'interleave = bsq'
byte_order = byte(256, 0)
printf, hdrf, 'byte order = ', byte_order
printf, hdrf, 'wavelength units = Micrometers'
;printf, hdrf, 'reflectance scale factor = 1.0'
;printf, hdrf, 'z plot range = {0.00, 1.00}
printf, hdrf, 'spectra names = {'
for i = 0, num_sp-2 do begin
  printf, hdrf, '  ', spNames[i], ','
endfor
printf, hdrf, '  ', spNames[num_sp-1], '}'
printf, hdrf, 'wavelength = {'
for i = 0, num_wvs-2 do begin
  printf, hdrf, '  ', wvs[i], ','
endfor
printf, hdrf, '  ', wvs[num_wvs-1], '}'
close, hdrf
free_lun, hdrf

end
